@@ -1,9 +1,10 @@
 # Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
 
 import copy
-import sys
+import functools
 from enum import Enum
-from typing import Any, Callable, Sequence, Tuple, Union
+from textwrap import dedent
+from typing import Any, Callable, Dict, List, Sequence, Tuple, Union
 
 from omegaconf import OmegaConf, SCMode
 from omegaconf._utils import is_structured_config
@@ -20,6 +21,7 @@ class _Keys(str, Enum):
     CONVERT = "_convert_"
     RECURSIVE = "_recursive_"
     ARGS = "_args_"
+    PARTIAL = "_partial_"
 
 
 def _is_target(x: Any) -> bool:
@@ -30,7 +32,7 @@ def _is_target(x: Any) -> bool:
     return False
 
 
-def _extract_pos_args(*input_args: Any, **kwargs: Any) -> Tuple[Any, Any]:
+def _extract_pos_args(input_args: Any, kwargs: Any) -> Tuple[Any, Any]:
     config_args = kwargs.pop(_Keys.ARGS, ())
     output_args = config_args
 
@@ -39,16 +41,16 @@ def _extract_pos_args(*input_args: Any, **kwargs: Any) -> Tuple[Any, Any]:
             output_args = input_args
     else:
         raise InstantiationException(
-            f"Unsupported _args_ type: {type(config_args).__name__}. value: {config_args}"
+            f"Unsupported _args_ type: '{type(config_args).__name__}'. value: '{config_args}'"
         )
 
     return output_args, kwargs
 
 
-def _call_target(_target_: Callable, *args, **kwargs) -> Any:  # type: ignore
+def _call_target(_target_: Callable, _partial_: bool, args, kwargs, full_key) -> Any:  # type: ignore
     """Call target (type) with args and kwargs."""
     try:
-        args, kwargs = _extract_pos_args(*args, **kwargs)
+        args, kwargs = _extract_pos_args(args, kwargs)
         # detaching configs from parent.
         # At this time, everything is resolved and the parent link can cause
         # issues when serializing objects in some scenarios.
@@ -58,12 +60,35 @@ def _call_target(_target_: Callable, *args, **kwargs) -> Any:  # type: ignore
         for v in kwargs.values():
             if OmegaConf.is_config(v):
                 v._set_parent(None)
-
-        return _target_(*args, **kwargs)
     except Exception as e:
-        raise type(e)(
-            f"Error instantiating '{_convert_target_to_string(_target_)}' : {e}"
-        ).with_traceback(sys.exc_info()[2])
+        msg = (
+            f"Error in collecting args and kwargs for '{_convert_target_to_string(_target_)}':"
+            + f"\n{repr(e)}"
+        )
+        if full_key:
+            msg += f"\nfull_key: {full_key}"
+
+        raise InstantiationException(msg) from e
+
+    if _partial_:
+        try:
+            return functools.partial(_target_, *args, **kwargs)
+        except Exception as e:
+            msg = (
+                f"Error in creating partial({_convert_target_to_string(_target_)}, ...) object:"
+                + f"\n{repr(e)}"
+            )
+            if full_key:
+                msg += f"\nfull_key: {full_key}"
+            raise InstantiationException(msg) from e
+    else:
+        try:
+            return _target_(*args, **kwargs)
+        except Exception as e:
+            msg = f"Error in call to target '{_convert_target_to_string(_target_)}':\n{repr(e)}"
+            if full_key:
+                msg += f"\nfull_key: {full_key}"
+            raise InstantiationException(msg) from e
 
 
 def _convert_target_to_string(t: Any) -> Any:
@@ -75,7 +100,7 @@ def _convert_target_to_string(t: Any) -> Any:
         return t
 
 
-def _prepare_input_dict(d: Any) -> Any:
+def _prepare_input_dict_or_list(d: Union[Dict[Any, Any], List[Any]]) -> Any:
     res: Any
     if isinstance(d, dict):
         res = {}
@@ -83,13 +108,13 @@ def _prepare_input_dict(d: Any) -> Any:
             if k == "_target_":
                 v = _convert_target_to_string(d["_target_"])
             elif isinstance(v, (dict, list)):
-                v = _prepare_input_dict(v)
+                v = _prepare_input_dict_or_list(v)
             res[k] = v
     elif isinstance(d, list):
         res = []
         for v in d:
             if isinstance(v, (list, dict)):
-                v = _prepare_input_dict(v)
+                v = _prepare_input_dict_or_list(v)
             res.append(v)
     else:
         assert False
@@ -97,18 +122,23 @@ def _prepare_input_dict(d: Any) -> Any:
 
 
 def _resolve_target(
-    target: Union[str, type, Callable[..., Any]]
+    target: Union[str, type, Callable[..., Any]], full_key: str
 ) -> Union[type, Callable[..., Any]]:
     """Resolve target string, type or callable into type or callable."""
     if isinstance(target, str):
-        return _locate(target)
-    if isinstance(target, type):
-        return target
-    if callable(target):
-        return target
-    raise InstantiationException(
-        f"Unsupported target type: {type(target).__name__}. value: {target}"
-    )
+        try:
+            target = _locate(target)
+        except Exception as e:
+            msg = f"Error locating target '{target}', see chained exception above."
+            if full_key:
+                msg += f"\nfull_key: {full_key}"
+            raise InstantiationException(msg) from e
+    if not callable(target):
+        msg = f"Expected a callable target, got '{target}' of type '{type(target).__name__}'"
+        if full_key:
+            msg += f"\nfull_key: {full_key}"
+        raise InstantiationException(msg)
+    return target
 
 
 def instantiate(config: Any, *args: Any, **kwargs: Any) -> Any:
@@ -128,7 +158,8 @@ def instantiate(config: Any, *args: Any, **kwargs: Any) -> Any:
                                   the exception of Structured Configs (and their fields).
                         all     : Passed objects are dicts, lists and primitives without
                                   a trace of OmegaConf containers
-                   _args_: List-like of positional arguments
+                   _partial_: If True, return functools.partial wrapped method or object
+                              False by default. Configure per target.
     :param args: Optional positional parameters pass-through
     :param kwargs: Optional named parameters to override
                    parameters in the config object. Parameters not present
@@ -147,17 +178,23 @@ def instantiate(config: Any, *args: Any, **kwargs: Any) -> Any:
     if isinstance(config, TargetConf) and config._target_ == "???":
         # Specific check to give a good warning about failure to annotate _target_ as a string.
         raise InstantiationException(
-            f"Missing value for {type(config).__name__}._target_. Check that it's properly annotated and overridden."
-            f"\nA common problem is forgetting to annotate _target_ as a string : '_target_: str = ...'"
+            dedent(
+                f"""\
+                Config has missing value for key `_target_`, cannot instantiate.
+                Config type: {type(config).__name__}
+                Check that the `_target_` key in your dataclass is properly annotated and overridden.
+                A common problem is forgetting to annotate _target_ as a string : '_target_: str = ...'"""
+            )
         )
+        # TODO: print full key
 
-    if isinstance(config, dict):
-        config = _prepare_input_dict(config)
+    if isinstance(config, (dict, list)):
+        config = _prepare_input_dict_or_list(config)
 
-    kwargs = _prepare_input_dict(kwargs)
+    kwargs = _prepare_input_dict_or_list(kwargs)
 
     # Structured Config always converted first to OmegaConf
-    if is_structured_config(config) or isinstance(config, dict):
+    if is_structured_config(config) or isinstance(config, (dict, list)):
         config = OmegaConf.structured(config, flags={"allow_objects": True})
 
     if OmegaConf.is_dict(config):
@@ -176,11 +213,42 @@ def instantiate(config: Any, *args: Any, **kwargs: Any) -> Any:
 
         _recursive_ = config.pop(_Keys.RECURSIVE, True)
         _convert_ = config.pop(_Keys.CONVERT, ConvertMode.NONE)
+        _partial_ = config.pop(_Keys.PARTIAL, False)
 
-        return instantiate_node(config, *args, recursive=_recursive_, convert=_convert_)
+        return instantiate_node(
+            config, *args, recursive=_recursive_, convert=_convert_, partial=_partial_
+        )
+    elif OmegaConf.is_list(config):
+        # Finalize config (convert targets to strings, merge with kwargs)
+        config_copy = copy.deepcopy(config)
+        config_copy._set_flag(
+            flags=["allow_objects", "struct", "readonly"], values=[True, False, False]
+        )
+        config_copy._set_parent(config._get_parent())
+        config = config_copy
+
+        OmegaConf.resolve(config)
+
+        _recursive_ = kwargs.pop(_Keys.RECURSIVE, True)
+        _convert_ = kwargs.pop(_Keys.CONVERT, ConvertMode.NONE)
+        _partial_ = kwargs.pop(_Keys.PARTIAL, False)
+
+        if _partial_:
+            raise InstantiationException(
+                "The _partial_ keyword is not compatible with top-level list instantiation"
+            )
+
+        return instantiate_node(
+            config, *args, recursive=_recursive_, convert=_convert_, partial=_partial_
+        )
     else:
         raise InstantiationException(
-            "Top level config has to be OmegaConf DictConfig, plain dict, or a Structured Config class or instance"
+            dedent(
+                f"""\
+                Cannot instantiate config of type {type(config).__name__}.
+                Top level config must be an OmegaConf DictConfig/ListConfig object,
+                a plain dict/list, or a Structured Config class or instance."""
+            )
         )
 
 
@@ -200,6 +268,7 @@ def instantiate_node(
     *args: Any,
     convert: Union[str, ConvertMode] = ConvertMode.NONE,
     recursive: bool = True,
+    partial: bool = False,
 ) -> Any:
     # Return None if config is None
     if node is None or (OmegaConf.is_config(node) and node._is_none()):
@@ -214,9 +283,21 @@ def instantiate_node(
         # if the key type is incompatible on get.
         convert = node[_Keys.CONVERT] if _Keys.CONVERT in node else convert
         recursive = node[_Keys.RECURSIVE] if _Keys.RECURSIVE in node else recursive
+        partial = node[_Keys.PARTIAL] if _Keys.PARTIAL in node else partial
+
+    full_key = node._get_full_key(None)
 
     if not isinstance(recursive, bool):
-        raise TypeError(f"_recursive_ flag must be a bool, got {type(recursive)}")
+        msg = f"Instantiation: _recursive_ flag must be a bool, got {type(recursive)}"
+        if full_key:
+            msg += f"\nfull_key: {full_key}"
+        raise TypeError(msg)
+
+    if not isinstance(partial, bool):
+        msg = f"Instantiation: _partial_ flag must be a bool, got {type( partial )}"
+        if node and full_key:
+            msg += f"\nfull_key: {full_key}"
+        raise TypeError(msg)
 
     # If OmegaConf list, create new list of instances if recursive
     if OmegaConf.is_list(node):
@@ -235,9 +316,9 @@ def instantiate_node(
             return lst
 
     elif OmegaConf.is_dict(node):
-        exclude_keys = set({"_target_", "_convert_", "_recursive_"})
+        exclude_keys = set({"_target_", "_convert_", "_recursive_", "_partial_"})
         if _is_target(node):
-            _target_ = _resolve_target(node.get(_Keys.TARGET))
+            _target_ = _resolve_target(node.get(_Keys.TARGET), full_key)
             kwargs = {}
             for key, value in node.items():
                 if key not in exclude_keys:
@@ -246,7 +327,8 @@ def instantiate_node(
                             value, convert=convert, recursive=recursive
                         )
                     kwargs[key] = _convert_node(value, convert)
-            return _call_target(_target_, *args, **kwargs)
+
+            return _call_target(_target_, partial, args, kwargs, full_key)
         else:
             # If ALL or PARTIAL non structured, instantiate in dict and resolve interpolations eagerly.
             if convert == ConvertMode.ALL or (
